## Niclas Schlapmann - Freelance 3D Technical Artist
## www.enoni.de
## hello@enoni.de
## 07.04.2024
##################################### Imports ####################################
import hou
import os
import sys
import traceback
import json
import getpass
import time
from time import *
##################################################################################

user = getpass.getuser()
lt = localtime()
year, month, day, hour, minute, sec = lt[0:6]
date = str(year)[2:4] + "-" + str(month).zfill(2) + "-" + str(day).zfill(2) + " - " + str(hour).zfill(2) + ":" + str(minute).zfill(2) + ":" + str(sec).zfill(2)

def deadline_submitter_husk_task_rs():
    renderNodes = hou.selectedNodes()
    
    if not renderNodes:
        return
    
    if renderNodes[0].type().name() not in ["usdrender_rop", "usd_rop"]:
        hou.ui.displayMessage("Select a proper USD-ROP or USDRender-ROP")
        return
    
    for renderNode in renderNodes:
        jobname = hou.getenv("HIPNAME")
        pool = "husk_redshift"
        secondarypool = "husk_redshift"
        comment = "submitted by <" + user + "> " + date
        department = "enoni.de"
        
        if renderNode.evalParm(renderNode.path() + "/trange") >= 1:
            framelist = str(int(renderNode.evalParm(renderNode.path() + "/f1"))) + "-" + str(int(renderNode.evalParm(renderNode.path() + "/f2")))
        else:
            framelist = str(int(hou.frame())) + "-" + str(int(hou.frame()))

        framecount = int(renderNode.evalParm(renderNode.path() + "/f2")) - int(renderNode.evalParm(renderNode.path() + "/f1")) + 1
        frame_input_count = hou.ui.readInput("Frames per task:", buttons=("OK", "Cancel"), initial_contents=str(framecount))

        if frame_input_count[0] == 1:
            return
        
        print("ns_Pipe> Render ROP: " + renderNode.name())
        
        choice = hou.ui.displayMessage("Deadline Priority?", buttons=("Extra Low(1)", "Low(25)", "Mid(50)", "High(99)", "Abort"))
        
        if choice == 0:
            prio = 1
        elif choice == 1:
            prio = 25
        elif choice == 2:
            prio = 50
        elif choice == 3:
            prio = 99
        else:
            return
        
        ## create prop dictionary ##
        jobProperties = {
            'batch': False,
            'jobname': jobname,
            'comment': comment,
            'department': department,
            'pool': pool,
            'secondarypool': secondarypool,
            'group': 'none',
            'priority': prio,
            'tasktimeout': 0,
            'autotimeout': 0,
            'concurrent': 1,
            'machinelimit': 0,
            'slavelimit': 1,
            'limits': '',
            'onjobcomplete': 'Nothing',
            'jobsuspended': 0,
            'shouldprecache': 1,
            'isblacklist': 0,
            'machinelist': '',
            'overrideframes': 1,
            'framelist': framelist,
            'framespertask': int(frame_input_count[1]),
            'bits': '64bit',
            'submitscene': 0,
            'isframedependent': 0,
            'gpuopenclenable': 0,
            'gpuspertask': 0,
            'gpudevices': '',
            'ignoreinputs': 0,
            'separateWedgeJobs': 0,
            
            'mantrajob': 0,
            'mantrapool': pool,
            'mantrasecondarypool': secondarypool,
            'mantragroup': 'none',
            'mantrapriority': prio,
            'mantratasktimeout': 0,
            'mantraautotimeout': 0,
            'mantraconcurrent': 1,
            'mantramachinelimit': 0,
            'mantraslavelimit': 1,
            'mantralimits': '',
            'mantraonjobcomplete': 'Nothing',
            'mantraisblacklist': 0,
            'mantramachinelist': '',
            'mantrathreads': 0,
            'mantralocalexport': 0,
            
            'arnoldjob': 1,
            'arnoldpool': pool,
            'arnoldsecondarypool': secondarypool,
            'arnoldgroup': 'none',
            'arnoldpriority': prio,
            'arnoldtasktimeout': 0,
            'arnoldautotimeout': 0,
            'arnoldconcurrent': 1,
            'arnoldmachinelimit': 0,
            'arnoldslavelimit': 1,
            'arnoldonjobcomplete': 'Nothing',
            'arnoldlimits': '',
            'arnoldisblacklist': 0,
            'arnoldmachinelist': '',
            'arnoldthreads': 0,
            'arnoldlocalexport': 1,
            
            'rendermanjob': 0,
            'rendermanpool': pool,
            'rendermansecondarypool': secondarypool,
            'rendermangroup': 'none',
            'rendermanpriority': prio,
            'rendermantasktimeout': 0,
            'rendermanconcurrent': 1,
            'rendermanmachinelimit': 0,
            'rendermanlimits': '',
            'rendermanonjobcomplete': 'Nothing',
            'rendermanisblacklist': 0,
            'rendermanmachinelist': '',
            'rendermanthreads': 0,
            'rendermanarguments': '',
            'rendermanlocalexport': 0,
            
            'redshiftjob': 0,
            'redshiftpool': pool,
            'redshiftsecondarypool': secondarypool,
            'redshiftgroup': 'none',
            'redshiftpriority': prio,
            'redshifttasktimeout': 0,
            'redshiftautotimeout': 0,
            'redshiftconcurrent': 1,
            'redshiftmachinelimit': 0,
            'redshiftslavelimit': 1,
            'redshiftlimits': '',
            'redshiftonjobcomplete': 'Nothing',
            'redshiftisblacklist': 0,
            'redshiftmachinelist': '',
            'redshiftarguments': '',
            'redshiftlocalexport': 0,
            
            'usdjob': 1,
            'usdpool': pool,
            'usdsecondarypool': secondarypool,
            'usdgroup': 'none',
            'usdpriority': prio,
            'usdtasktimeout': 0,
            'usdautotimeout': 0,
            'usdconcurrent': 1,
            'usdmachinelimit': 0,
            'usdslavelimit': 1,
            'usdlimits': '',
            'usdonjobcomplete': 'Nothing',
            'usdisblacklist': 0,
            'usdmachinelist': '',
            'usdarguments': '',
            'usdlocalexport': 1,
            'usdloglevel': 2,
    
            'vrayjob': 0,
            'vraypool': pool,
            'vraysecondarypool': secondarypool,
            'vraygroup': 'none',
            'vraypriority': prio,
            'vraytasktimeout': 0,
            'vrayautotimeout': 0,
            'vrayconcurrent': 1,
            'vraymachinelimit': 0,
            'vrayslavelimit': 1,
            'vraylimits': '',
            'vrayonjobcomplete': 'Nothing',
            'vrayisblacklist': 0,
            'vraymachinelist': '',
            'vraythreads': 0,
            'vrayarguments': '',
            'vraylocalexport': 0,
            
            'tilesenabled': 0,
            'tilesinx': 3,
            'tilesiny': 3,
            'tilessingleframeenabled': 1,
            'tilessingleframe': 1,
            'jigsawenabled': 1,
            'jigsawregioncount': 0,
            'jigsawregions': [],
            'submitdependentassembly': 1,
            'backgroundoption': 'Blank Image',
            'backgroundimage': '',
            'erroronmissingtiles': '1',
            'erroronmissingbackground': '0',
            'cleanuptiles': '1'
        }
        
        ## write USD from USD-ROP ##
        usd_file_path = renderNode.evalParm(renderNode.path() + "/lopoutput")
        if os.path.isfile(usd_file_path):
            if hou.ui.displayMessage("USD render file already exist. Override", buttons=("Yes", "Abort")) == 0:
                renderNode.parm("execute").pressButton()
            else:
                return
        else:
            renderNode.parm("execute").pressButton()

        tmp_version = ""
        
        ## submit to Deadline ##
        flag = 0
        
        ## imports and sys pathes for deadline ##
        try:
            from CallDeadlineCommand import CallDeadlineCommand
        except ImportError:
            path = ""
            print("The CallDeadlineCommand.py script could not be found in the Houdini installation. Please make sure that the Deadline Client has been installed on this machine.\n")
            hou.ui.displayMessage("The CallDeadlineCommand.py script could not be found in the Houdini installation. Please make sure that the Deadline Client has been installed on this machine.", title="Submit Houdini To Deadline")
        else:
            path = CallDeadlineCommand(["-GetRepositoryPath", "submission/Houdini/Main"]).strip()
        
        if path:
            path = path.replace("\\", "/")
            
            # Add the path to the system path
            if path not in sys.path:
                print("Appending \"" + path + "\" to system path to import SubmitHoudiniToDeadline module")
                sys.path.append(path)
            else:
                pass
            
            # Import the script and call the main() function
            try:
                import SubmitHoudiniToDeadline
            except:
                print(traceback.format_exc())
                print("The SubmitHoudiniToDeadline.py script could not be found in the Deadline Repository. Please make sure that the Deadline Client has been installed on this machine, that the Deadline Client bin folder is set in the DEADLINE_PATH environment variable, and that the Deadline Client has been configured to point to a valid Repository.")
        else:
            print("The SubmitHoudiniToDeadline.py script could not be found in the Deadline Repository. Please make sure that the Deadline Client has been installed on this machine, that the Deadline Client bin folder is set in the DEADLINE_PATH environment variable, and that the Deadline Client has been configured to point to a valid Repository.")
        
        ## get deadline info ##
        print("Grabbing submitter info...")
        try:
            output = json.loads(CallDeadlineCommand(["-prettyJSON", "-GetSubmissionInfo", "Pools", "Groups", "MaxPriority", "TaskLimit", "UserHomeDir", "RepoDir:submission/Houdini/Main", "RepoDir:submission/Integration/Main", "RepoDirNoCustom:draft",
                                                     "RepoDirNoCustom:submission/Jigsaw", ]))
        except:
            print("Unable to get submitter info from Deadline:\n\n" + traceback.format_exc())
            raise
        
        if output["ok"]:
            submissionInfo = output["result"]
            hou.putenv("Deadline_Submission_Info", json.dumps(submissionInfo))
        else:
            print("DeadlineCommand returned a bad result and was unable to grab the submitter info.\n\n" + output["result"])
            raise Exception(output["result"])
        
        ## submit render job ##
        try:
            import SubmitHoudiniToDeadlineFunctions as SHTDFunctions
            flag = 1
        except Exception as e:
            print(e)
            hou.ui.displayMessage("ns_Pipe> Library import failure. Make sure you have a proper Deadline installation.")
        
        if flag:
            try:
                jobProperties.update({"ns_delegate" : "Redshift"})
                jobProperties.update({"ns_package" : hou.getenv("HOUDINI_PACKAGE_DIR")})
                jobProperties.update({"ns_pipe_hou_version": tmp_version})
                jobProperties.update({"ns_pipe_image_out" : hou.parm(renderNode.path() + "/spare_output_image").eval()})
                jobIds = SHTDFunctions.SubmitRenderJob_husk(renderNode, jobProperties, "")
            except Exception as e:
                print(e)
                hou.ui.displayMessage("ns_Pipe> Can`t submitting to Deadline Repository.")

deadline_submitter_husk_task_rs()